/*
    RGB array generator for RGB led controler testing.
    Copyright (C) 2015  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>

/***************************************************************************/
/* define vari */

/***************************************************************************/
/* macro */

//#define Sum(C,D)	(unsigned char)((float)C + D)

/***************************************************************************/
/* strutture */

typedef struct
{
  unsigned char		Red;
  unsigned char		Green;
  unsigned char		Blue;
}Color_t;

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"colors version 0.01, Copyright (C) 2015  Alessandro Accattini\n";
static const char Help[]=
	"Usage: colors <options> [r1,g1,b1] [r2,g2,b2]\n"
	"generates a RGB array for RGB led controller.\n\n"
	"\t-1\toutputs only the first half of the registers array.\n"
	"\t-2\toutputs only the second half of the registers array.\n"
	"\t-h\tdisplays this help.\n"
	"Report bugs to bit.trapper@gmail.com\n";

/***************************************************************************/
/* variabili */

/***************************************************************************/
/* prototipi funzioni */

static unsigned char Sum(unsigned char Color,float Delta);
  /* Somma il delta al colore indicato. */

static int ArrCoord(int x,int y);
  /* Converte le coordinate del pcb in un indica per l'array RGB. */

static void RgbIns(const Color_t *Color,unsigned short int *Rgb,int ArrCoord);
  /* Inserisce una tripletta RGB nell'array RGB. */
  
static void Diagonal(const Color_t *Color1,const Color_t *Color2,
		     int From,int To);
  /* Realizza una sfumatura a bande diagonali. */
  
static int ReadColor(const char *Arg,Color_t *ColorPtr);
  /* Legge e traduce un descrittore di colore, ritorna 0 in caso di
     successo. */

/***************************************************************************/
/* funzioni */

static unsigned char Sum(unsigned char Color,float Delta)
{
  /* Somma il delta al colore indicato. */



  float		Fcolor=(float)Color;


  Fcolor+=Delta;				// somma
  if(Fcolor > 255.0)				// limita
    Fcolor=255.0;
  if(Fcolor < 0.0)
    Fcolor=0.0;
  return((unsigned char)Fcolor);		// termina
}



static int ArrCoord(int x,int y)
{
  /* Converte le coordinate del pcb in un indica per l'array RGB. */



  if(x & 1)				// se x e' dispari
    return((x<<2)+(3-y));		// x*4+(3-y)
  return((x<<2)+y);			// x*4+y
}



static void RgbIns(const Color_t *Color,unsigned short int *Rgb,int ArrCoord)
{
  /* Inserisce una tripletta RGB nell'array RGB. */



  int	Idx;



  if(ArrCoord & 1)			// se l'indice e' dispari
  {
    // indice dispari
    Idx=(((ArrCoord-1)>>1)*3)+2;	// word intera
    Rgb[Idx]=	(((unsigned short int)Color->Blue)<<8)|
  		((unsigned short int)Color->Red);
    Idx--;				// prev half word
    Rgb[Idx]&=0x00ff;			// elimina la parte alta
    Rgb[Idx]|=	((unsigned short int)Color->Green)<<8;
    return;				// termina
  }

  // indica pari
  Idx=(ArrCoord>>1)*3;			// word intera
  Rgb[Idx]=	(((unsigned short int)Color->Red)<<8)|
  		((unsigned short int)Color->Green);
  Idx++;				// next half word
  Rgb[Idx]&=0xff00;			// elimina la parte bassa
  Rgb[Idx]|=	(unsigned short int)Color->Blue;
  return;				// termina
}



static void Diagonal(const Color_t *Color1,const Color_t *Color2,
		     int From,int To)
{
  /* Realizza una sfumatura a bande diagonali. */



  Color_t		Color=*Color1;
  float			Dr,Dg,Db;
  int			i;
  unsigned short int	RgbArray[3*8];



  // calcola i delta
  Dr=((float)Color2->Red - (float)Color1->Red)/6.0;
  Dg=((float)Color2->Green - (float)Color1->Green)/6.0;
  Db=((float)Color2->Blue - (float)Color1->Blue)/6.0;

  // colora i pixel
  RgbIns(&Color,RgbArray,ArrCoord(0,0));		// angolo iniziale
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(0,1));
  RgbIns(&Color,RgbArray,ArrCoord(1,0));
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(0,2));
  RgbIns(&Color,RgbArray,ArrCoord(1,1));
  RgbIns(&Color,RgbArray,ArrCoord(2,0));
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(0,3));		// diagonale
  RgbIns(&Color,RgbArray,ArrCoord(1,2));
  RgbIns(&Color,RgbArray,ArrCoord(2,1));
  RgbIns(&Color,RgbArray,ArrCoord(3,0));
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(1,3));
  RgbIns(&Color,RgbArray,ArrCoord(2,2));
  RgbIns(&Color,RgbArray,ArrCoord(3,1));
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(2,3));
  RgbIns(&Color,RgbArray,ArrCoord(3,2));
  Color.Red=Sum(Color.Red,Dr);
  Color.Green=Sum(Color.Green,Dg);
  Color.Blue=Sum(Color.Blue,Db);
  RgbIns(&Color,RgbArray,ArrCoord(3,3));		// angolo finale

  // stampa l'array RGB
  for(i=From;i<To;i++)
    printf("0x%04x ",RgbArray[i]);
  return;					// termina
}



static int ReadColor(const char *Arg,Color_t *ColorPtr)
{
  /* Legge e traduce un descrittore di colore, ritorna 0 in caso di
     successo. */



  unsigned int	Red,Green,Blue;


  
  if(sscanf(Arg,"[%i,%i,%i]",&Red,&Green,&Blue) == 3)
  {
    ColorPtr->Red=(unsigned char)Red;
    ColorPtr->Green=(unsigned char)Green;
    ColorPtr->Blue=(unsigned char)Blue;
    return 0;					// termina con successo
  }  
  fputs("Bad color descriptor format.\n",stderr);
  return 1;					// termina con fallimento
}





/***************************************************************************/

int main(int argc,char **argv)
{
  Color_t	Color1,Color2;		// colore iniziale e finale
  int		Opzione;		// per la lettura delle opzioni
  int		From=0;			// inizio parte stampata
  int		To=3*16/2;		// fine parte stampata



  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"12h"))!=-1)
  {
    switch(Opzione)
    {
      case '1':
        From=0;				// prima meta'
	To=3*4;
      break;
      case '2':
        From=3*4;			// seconda meta'
	To=3*8;
      break;
      case 'h':
	puts(Licenza);			// licenza
        puts(Help);			// stampa l'help
	return 0;			// termina
      break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
      break;
    }
  }
  
//  // processa tutti gli atri parametri passati
//  for(;optind<argc;++optind)
//  {
//    printf("processing :%s \n",argv[optind]);
//  }

  // verifica di avere parametri a sufficienza
  if((argc-optind) < 1)			// se manca roba
  {
    puts(Licenza);			// licenza
    puts(Help);				// stampa l'help
    return 0;				// termina
  }

  // legge il colore iniziale
  if(ReadColor(argv[optind],&Color1))	// legge il colore 1
    return 1;				// termina con fallimento

  // legge il colore finale
  if((argc-optind) >= 2)		// se c'e' anche il secondo
  {
    optind++;				// next argument
    if(ReadColor(argv[optind],&Color2))	// legge il colore 2
      return 1;				// termina con fallimento
  }
  else
  {
    Color2=Color1;			// copia il primo colore
  }

/*
  // stampa i colori per verifica
  printf("Color1:\n");
  printf("  R:0x%02x\n",(unsigned int)Color1.Red);
  printf("  G:0x%02x\n",(unsigned int)Color1.Green);
  printf("  B:0x%02x\n",(unsigned int)Color1.Blue);
  printf("Color2:\n");
  printf("  R:0x%02x\n",(unsigned int)Color2.Red);
  printf("  G:0x%02x\n",(unsigned int)Color2.Green);
  printf("  B:0x%02x\n",(unsigned int)Color2.Blue);
*/
  
  // realizza la sfumatura
  Diagonal(&Color1,&Color2,From,To);	// sfumatura diagonale
  return 0;				// termina con successo
}

/* $Source$ */

