;
;   WS2812B based RGB led controller.
;   Copyright (C) 2015 2017  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
; 0.01	First version, tested only on AT90S2313.
; 0.02  Adottata nuova libreria modbus.
;	Provvisoriamente pilota solo i led collegati alla porta B.
; 0.03	Pilota fino a 14 gruppi di 16 led ciascuno, distribuiti tra le porte
;	B e D.
;
;

.include "2313def.inc"			;AT90S2313
.equ		Clock8		=1	;quarzo da 8.0000MHz



.CSEG
;vettori di interruzione
		.org 0x0000
		rjmp Reset		;reset
		rjmp UnusedInt		;external interrupt 0
		rjmp UnusedInt		;external interrupt 1
		rjmp UnusedInt		;timer 1 capture
		rjmp UnusedInt		;timer 1 compare match
		rjmp UnusedInt		;timer 1 overflow
		rjmp IntTimer0		;timer 0 overflow
		rjmp IntRx		;uart RX complete
		rjmp IntTx		;uart data register empty
		rjmp IntTxComplete	;uart TX complete
		rjmp UnusedInt		;analog comparator




.DSEG

		.org 0x0060		;inizio RAM
;inizio registri modbus
CtrlReg:	.byte 2			;registro di controllo
					;HIGH=portB, LOW=portD, little endian
ModReg:		.byte 3*16		;array RGB
;fine registri modbus





.include "define.asm"			;define vari
.include "modbus-0.03.asm"		;ModBus protocol
.include "ws2812b.asm"			;RGB led control





.CSEG


UnusedInt:	nop			;serve l'interrupt
		reti			;termina l'interrupt

;inizializzazione del micro
Reset:		cli			;disabilita le interruzioni
		ldi r16,low(Stack)	;punta lo stack in fondo alla RAM
		out SPL,r16
;inizializza l'array RGB
		ldi YH,high(ModReg)	;punta Y all'array di registri
		ldi YL,low(ModReg)
		clr r16			;azzera l'array GRB
		ldi r17,3*16		;carica il contatore dei led
InitLoop:	st Y+,r16		;clear ALL
		dec r17			;scala il contatore
		brne InitLoop		;se non zero salta
;inizializza il sistema
		rcall WS2812_Init	;I/O init
		ldi r16,0x01		;in r16 il device address
		rcall ModbusInit	;inizializza il protocollo Modbus
;abilita le interruzioni
		sei			;abilita le interruzioni
;spegne TUTTI i led
		ldi YH,high(ModReg)	;punta Y all'array RGB
		ldi YL,low(ModReg)
		ldi XH,high(3*16)	;in X il numero di byte
		ldi XL,low(3*16)
		ldi r16,0xff		;in r16 la maschera delle linee di I/O
		rcall WS2812_Write_Port1	;spegne i led della porta B
		ldi YH,high(ModReg)	;punta Y all'array RGB
		ldi YL,low(ModReg)
		ldi XH,high(3*16)	;in X il numero di byte
		ldi XL,low(3*16)
		ldi r16,0xfc		;in r16 la maschera delle linee di I/O
		rcall WS2812_Write_Port2	;spegne i led della porta D
;attende un messaggio dalla seriale
WaitCmd:	
		rcall ModbusSlaveLoop	;esegue i comandi in arrivo
		rjmp WaitCmd



ModBus_03_Callback:			;callback lettura registri
		ret			;termina


ModBus_06_Callback:
ModBus_16_Callback:

WriteCallback:
;Chiamata dopo ogni scrittura da parte del master remoto, si occupa di rendere
;attive le modifiche sui led RGB.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	tutti i registri vengono conservati
		push r16		;salva i registri utilizzati
		lds r16,CtrlReg+0	;in r16 la parte bassa del reg. comandi
		bst r16,0		;in T il bit 0
		andi r16,0xfc		;elimina i due bit piu' bassi
		sts CtrlReg+0,r16	;aggiorna il registro
		brts LedUpdate		;se T=1 salta
WriteExit:	pop r16			;recupera i registri
		ret			;termina
;il master remoto ha settato il flag 'led update'		
LedUpdate:	push YH			;salva i registri utilizzati
		push YL
		push XH
		push XL
;aggiorna i led collegati alla porta D
		tst r16			;test del valore in r16
		breq UpdatePortB	;se zero salta
		ldi YH,high(ModReg)	;punta Y all'array RGB
		ldi YL,low(ModReg)
		ldi XH,high(3*16)	;in X il numero di byte
		ldi XL,low(3*16)
		rcall WS2812_Write_Port2	;aggiorna i led
;aggiorna i led collegati alla porta B
UpdatePortB:	lds r16,CtrlReg+1	;in r16 la parte alta del reg. di ctrl
		tst r16			;test del valore in r16
		breq SkipUpdate		;se zero salta
		ldi YH,high(ModReg)	;punta Y all'array RGB
		ldi YL,low(ModReg)
		ldi XH,high(3*16)	;in X il numero di byte
		ldi XL,low(3*16)
		rcall WS2812_Write_Port1	;aggiorna i led
SkipUpdate:	pop XL			;recupera i registri
		pop XH
		pop YL
		pop YH
		rjmp WriteExit

		
