;
;   WS2812B RGB led driver control routine.
;   Copyright (C) 2015 2017  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
;
;

;*****************************************************************************
;  Required settings
; This routine requires 8.00MHz clock.
;

;*****************************************************************************
;  Register mapping

.def	Data		=r16		;data byte
.def	LineHigh	=r17		;usato per alzare la linea
.def	LineLow		=r18		;usato per abbassare la linea
.def	BitCount	=r19		;contatore dei bit

;*****************************************************************************
;  I/O settings

.equ	WS2812_PORT		=PORTB
.equ	WS2812_DDR		=DDRB
.equ	WS2812_PORT_MASK	=0xff	;all bits of port B

.equ	WS2812_PORT2		=PORTD
.equ	WS2812_DDR2		=DDRD
.equ	WS2812_PORT2_MASK	=0xfc	;6 higher bits of port D

;*****************************************************************************



.CSEG


WS2812_Init:
;Inizializza il pin di controllo al suo stato fondamentale.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	nessun registro viene garantito
;		sbi WS2812_DDR,WS2812_PIN	;pin in uscita
;		cbi WS2812_PORT,WS2812_PIN	;abbassa la linea
;configura la prima porta di uscita		
		in r16,WS2812_DDR		;in r16 la dir. della porta
		ori r16,WS2812_PORT_MASK	;linee utilizzate in uscita
		out WS2812_DDR,r16
		in r16,WS2812_PORT		;in r16 l'uscita della porta
		andi r16,~WS2812_PORT_MASK	;abbassa i pin utilizzati
		out WS2812_PORT,r16
;configura la seconda porta di uscita		
		in r16,WS2812_DDR2		;in r16 la dir. della porta
		ori r16,WS2812_PORT2_MASK	;linee utilizzate in uscita
		out WS2812_DDR2,r16
		in r16,WS2812_PORT2		;in r16 l'uscita della porta
		andi r16,~WS2812_PORT2_MASK	;abbassa i pin utilizzati
		out WS2812_PORT2,r16
		ret			;termina



WS2812_Write_Port1:
WS2812_Write:
;Aggiorna tutti i led RGB collegati alla prima porta di uscita.
;Per chiamare:	r16 contiene la maschera della linea dati dei led
;		Y punta al primo data byte da inviare
;		X contiene il numero di byte da inviare (>=1)
;In uscita:	Y punta al primo byte non prelevato
;		X viene perso
;		tutti i registri vengono conservati
		push Data		;salva i registri utilizzati
		push LineHigh
		push LineLow
		push BitCount
;prepara le maschere
		in LineHigh,WS2812_PORT	;legge lo stato della porta
		mov LineLow,LineHigh	;lo copia
;		ori LineHigh,1<<WS2812_PIN	;maschera per alzare
;		andi LineLow,~(1<<WS2812_PIN)	;maschera per abbassare
		or LineHigh,r16		;maschera per alzare
		com r16			;inverte la maschera
		and LineLow,r16		;maschera per abbassare
		ldi BitCount,6		;carica il contatore dei bit
		ld Data,Y+		;carica il primo data byte
		cli			;disabilita le interruzioni
;trasferimento bit da 7 a 2
WS2812_bit72:	out WS2812_PORT,LineHigh	;alza la linea
		rol Data		;shift data register
		brcs WS2812_Skip1	;skip se C=1
		out WS2812_PORT,LineLow	;abbassa la linea
WS2812_Skip1:	dec BitCount		;scala il contatore
		nop
		nop
		out WS2812_PORT,LineLow	;abbassa la linea
		brne WS2812_bit72	;se non zero salta
		nop
;trasferimento bit 1
		out WS2812_PORT,LineHigh	;alza la linea
		rol Data		;shift data register
		brcs WS2812_Skip2		;skip se C=1
		out WS2812_PORT,LineLow	;abbassa la linea
WS2812_Skip2:	sbiw XL,1		;scala il contatore dei byte
		nop
		out WS2812_PORT,LineLow	;abbassa la linea
		breq WS2812_LastBit	;se zero salta
		nop
;trasferimento bit 1
		out WS2812_PORT,LineHigh	;alza la linea
		rol Data		;shift data register
		brcs WS2812_Skip3	;skip se C=1
		out WS2812_PORT,LineLow	;abbassa la linea
WS2812_Skip3:	ld Data,Y+		;preleva un data byte
		ldi BitCount,6		;carica il contatore dei bit
		out WS2812_PORT,LineLow	;abbassa la linea
		rjmp WS2812_Bit72
;ultimo bit della sequenza
WS2812_LastBit:	out WS2812_PORT,LineHigh	;alza la linea
		rol Data		;shift data register
		brcs WS2812_Skip4	;skip se C=1
		out WS2812_PORT,LineLow	;abbassa la linea
WS2812_Skip4:	nop
		nop
		nop
		out WS2812_PORT,LineLow	;abbassa la linea
WS2812_Delay_b: sei			;abilita le interruzioni
;pausa di 400 tic, pari a 50us@8MHz
		ldi Data,100		;carica il contatore
WS2812_Delay:	nop
		dec Data
		brne WS2812_Delay	;se non zero salta
		pop BitCount		;recupera i registri
		pop LineLow
		pop LineHigh
		pop Data
		ret			;termina



WS2812_Write_Port2:
;Aggiorna tutti i led RGB collegati alla seconda porta di uscita.
;Per chiamare:	r16 contiene la maschera della linea dati dei led
;		Y punta al primo data byte da inviare
;		X contiene il numero di byte da inviare (>=1)
;In uscita:	Y punta al primo byte non prelevato
;		X viene perso
;		tutti i registri vengono conservati
		push Data		;salva i registri utilizzati
		push LineHigh
		push LineLow
		push BitCount
;prepara le maschere
		in LineHigh,WS2812_PORT2	;legge lo stato della porta
		mov LineLow,LineHigh	;lo copia
;		ori LineHigh,1<<WS2812_PIN	;maschera per alzare
;		andi LineLow,~(1<<WS2812_PIN)	;maschera per abbassare
		or LineHigh,r16		;maschera per alzare
		com r16			;inverte la maschera
		and LineLow,r16		;maschera per abbassare
		ldi BitCount,6		;carica il contatore dei bit
		ld Data,Y+		;carica il primo data byte
		cli			;disabilita le interruzioni
;trasferimento bit da 7 a 2
WS2812_bit72_b:	out WS2812_PORT2,LineHigh	;alza la linea
		rol Data			;shift data register
		brcs WS2812_Skip1_b		;skip se C=1
		out WS2812_PORT2,LineLow	;abbassa la linea
WS2812_Skip1_b:	dec BitCount			;scala il contatore
		nop
		nop
		out WS2812_PORT2,LineLow	;abbassa la linea
		brne WS2812_bit72_b		;se non zero salta
		nop
;trasferimento bit 1
		out WS2812_PORT2,LineHigh	;alza la linea
		rol Data			;shift data register
		brcs WS2812_Skip2_b		;skip se C=1
		out WS2812_PORT2,LineLow	;abbassa la linea
WS2812_Skip2_b:	sbiw XL,1			;scala il contatore dei byte
		nop
		out WS2812_PORT2,LineLow	;abbassa la linea
		breq WS2812_LastBit_b		;se zero salta
		nop
;trasferimento bit 1
		out WS2812_PORT2,LineHigh	;alza la linea
		rol Data			;shift data register
		brcs WS2812_Skip3_b		;skip se C=1
		out WS2812_PORT2,LineLow	;abbassa la linea
WS2812_Skip3_b:	ld Data,Y+			;preleva un data byte
		ldi BitCount,6			;carica il contatore dei bit
		out WS2812_PORT2,LineLow	;abbassa la linea
		rjmp WS2812_Bit72_b
;ultimo bit della sequenza
WS2812_LastBit_b:
		out WS2812_PORT2,LineHigh	;alza la linea
		rol Data			;shift data register
		brcs WS2812_Skip4_b		;skip se C=1
		out WS2812_PORT2,LineLow	;abbassa la linea
WS2812_Skip4_b:	nop
		nop
		nop
		out WS2812_PORT2,LineLow	;abbassa la linea
		rjmp WS2812_Delay_b


